@@ -7,35 +7,38 @@
 */
 
 #include "kded.h"
-#include "kdedadaptor.h"
 #include "kded_version.h"
+#include "kdedadaptor.h"
 
 #include <KCrash>
 
 #include <qplatformdefs.h>
 
-#include <QLoggingCategory>
-#include <QDir>
-#include <QCommandLineParser>
 #include <QApplication>
+#include <QCommandLineParser>
+#include <QDir>
+#include <QLoggingCategory>
 #include <QProcess>
 
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QDBusServiceWatcher>
 
-#include <KDBusService>
 #include <KConfigGroup>
-#include <KSharedConfig>
+#include <KDBusService>
 #include <KDirWatch>
-#include <KServiceTypeTrader>
+#include <KPluginFactory>
 #include <KPluginInfo>
 #include <KPluginMetaData>
+#include <KServiceTypeTrader>
+#include <KSharedConfig>
 
 #ifdef Q_OS_OSX
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
+#include <memory>
+
 Q_DECLARE_LOGGING_CATEGORY(KDED)
 
 Q_LOGGING_CATEGORY(KDED, "kf.kded", QtWarningMsg)
@@ -60,7 +63,7 @@ static void runKonfUpdate()
 }
 
 Kded::Kded()
-    : m_pDirWatch(nullptr)
+    : m_pDirWatch(new KDirWatch(this))
     , m_pTimer(new QTimer(this))
     , m_needDelayedCheck(false)
 {
@@ -69,8 +72,7 @@ Kded::Kded()
     m_serviceWatcher = new QDBusServiceWatcher(this);
     m_serviceWatcher->setConnection(QDBusConnection::sessionBus());
     m_serviceWatcher->setWatchMode(QDBusServiceWatcher::WatchForUnregistration);
-    QObject::connect(m_serviceWatcher, &QDBusServiceWatcher::serviceUnregistered,
-                     this, &Kded::slotApplicationRemoved);
+    QObject::connect(m_serviceWatcher, &QDBusServiceWatcher::serviceUnregistered, this, &Kded::slotApplicationRemoved);
 
     new KBuildsycocaAdaptor(this);
     new KdedAdaptor(this);
@@ -82,25 +84,20 @@ Kded::Kded()
     qDBusAddSpyHook(messageFilter);
 
     m_pTimer->setSingleShot(true);
-    connect(m_pTimer, &QTimer::timeout, this, static_cast<void(Kded::*)()>(&Kded::recreate));
+    connect(m_pTimer, &QTimer::timeout, this, static_cast<void (Kded::*)()>(&Kded::recreate));
 }
 
 Kded::~Kded()
 {
     _self = nullptr;
     m_pTimer->stop();
-    delete m_pTimer;
-    delete m_pDirWatch;
 
-    for (QHash<QString, KDEDModule *>::const_iterator
-            it(m_modules.constBegin()), itEnd(m_modules.constEnd());
-            it != itEnd; ++it) {
+    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
         KDEDModule *module(it.value());
 
         // first disconnect otherwise slotKDEDModuleRemoved() is called
         // and changes m_modules while we're iterating over it
-        disconnect(module, &KDEDModule::moduleDeleted,
-                   this, &Kded::slotKDEDModuleRemoved);
+        disconnect(module, &KDEDModule::moduleDeleted, this, &Kded::slotKDEDModuleRemoved);
 
         delete module;
     }
@@ -130,31 +127,32 @@ void Kded::messageFilter(const QDBusMessage &message)
 
 static int phaseForModule(const KPluginMetaData &module)
 {
-    const QVariant phasev = module.rawData().value(QStringLiteral("X-KDE-Kded-phase")).toVariant();
-    return phasev.isValid() ? phasev.toInt() : 2;
+    return module.value(QStringLiteral("X-KDE-Kded-phase"), 2);
 }
 
 QVector<KPluginMetaData> Kded::availableModules() const
 {
-    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
+    QVector<KPluginMetaData> plugins = KPluginMetaData::findPlugins(QStringLiteral("kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/kded"));
     QSet<QString> moduleIds;
-    for (const KPluginMetaData &md : qAsConst(plugins)) {
+    for (const KPluginMetaData &md : std::as_const(plugins)) {
         moduleIds.insert(md.pluginId());
     }
 #if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
     // also search for old .desktop based kded modules
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
     const KPluginInfo::List oldStylePlugins = KPluginInfo::fromServices(KServiceTypeTrader::self()->query(QStringLiteral("KDEDModule")));
-QT_WARNING_POP
+    QT_WARNING_POP
     for (const KPluginInfo &info : oldStylePlugins) {
         if (moduleIds.contains(info.pluginName())) {
-            qCWarning(KDED).nospace() << "kded module " << info.pluginName() << " has already been found using "
-                "JSON metadata, please don't install the now unneeded .desktop file (" << info.entryPath() << ").";
+            qCWarning(KDED).nospace() << "kded module " << info.pluginName()
+                                      << " has already been found using "
+                                         "JSON metadata, please don't install the now unneeded .desktop file ("
+                                      << info.entryPath() << ").";
         } else {
-            qCDebug(KDED).nospace() << "kded module " << info.pluginName() << " still uses .desktop files ("
-                << info.entryPath() << "). Please port it to JSON metadata.";
+            qCDebug(KDED).nospace() << "kded module " << info.pluginName() << " still uses .desktop files (" << info.entryPath()
+                                    << "). Please port it to JSON metadata.";
             plugins.append(info.toMetaData());
         }
     }
@@ -164,7 +162,7 @@ QT_WARNING_POP
 
 static KPluginMetaData findModule(const QString &id)
 {
-    KPluginMetaData module(QStringLiteral("kf5/kded/") + id);
+    KPluginMetaData module(QStringLiteral("kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/kded/") + id);
     if (module.isValid()) {
         return module;
     }
@@ -172,13 +170,13 @@ static KPluginMetaData findModule(const QString &id)
     // TODO KF6: remove the .desktop fallback code
     KService::Ptr oldStyleModule = KService::serviceByDesktopPath(QStringLiteral("kded/") + id + QStringLiteral(".desktop"));
     if (oldStyleModule) {
-        qCDebug(KDED).nospace() << "kded module " << oldStyleModule->desktopEntryName()
-            << " still uses .desktop files (" << oldStyleModule->entryPath() << "). Please port it to JSON metadata.";
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+        qCDebug(KDED).nospace() << "kded module " << oldStyleModule->desktopEntryName() << " still uses .desktop files (" << oldStyleModule->entryPath()
+                                << "). Please port it to JSON metadata.";
+        QT_WARNING_PUSH
+        QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+        QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
         return KPluginInfo(oldStyleModule).toMetaData();
-QT_WARNING_POP
+        QT_WARNING_POP
     }
 #endif
     qCWarning(KDED) << "could not find kded module with id" << id;
@@ -204,7 +202,7 @@ void Kded::initModules()
             kde_running = false;
         }
 #endif
-        //TODO: Change 5 to KDED_VERSION_MAJOR the moment KF5 are stable
+        // TODO: Change 5 to KDED_VERSION_MAJOR the moment KF5 are stable
         // not the same kde version as the current desktop
         const QByteArray kdeSession = qgetenv("KDE_SESSION_VERSION");
         if (kdeSession.toInt() != 5) {
@@ -294,7 +292,7 @@ bool Kded::isModuleAutoloaded(const KPluginMetaData &module) const
         return false;
     }
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
-    bool autoload = module.rawData().value(QStringLiteral("X-KDE-Kded-autoload")).toVariant().toBool();
+    bool autoload = module.value(QStringLiteral("X-KDE-Kded-autoload"), false);
     KConfigGroup cg(config, QStringLiteral("Module-").append(module.pluginId()));
     autoload = cg.readEntry("autoload", autoload);
     return autoload;
@@ -302,7 +300,7 @@ bool Kded::isModuleAutoloaded(const KPluginMetaData &module) const
 
 bool Kded::platformSupportsModule(const KPluginMetaData &module) const
 {
-    const QStringList supportedPlatforms = KPluginMetaData::readStringList(module.rawData(), QStringLiteral("X-KDE-OnlyShowOnQtPlatforms"));
+    const QStringList supportedPlatforms = module.value(QStringLiteral("X-KDE-OnlyShowOnQtPlatforms"), QStringList());
 
     return supportedPlatforms.isEmpty() || supportedPlatforms.contains(qApp->platformName());
 }
@@ -318,13 +316,7 @@ bool Kded::isModuleLoadedOnDemand(const KPluginMetaData &module) const
         return false;
     }
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
-    bool loadOnDemand = true;
-    // use toVariant() since it could be string or bool in the json and QJsonObject does not convert
-    QVariant p = module.rawData().value(QStringLiteral("X-KDE-Kded-load-on-demand")).toVariant();
-    if (p.isValid() && (p.toBool() == false)) {
-        loadOnDemand = false;
-    }
-    return loadOnDemand;
+    return module.value(QStringLiteral("X-KDE-Kded-load-on-demand"), true);
 }
 
 KDEDModule *Kded::loadModule(const QString &obj, bool onDemand)
@@ -355,9 +347,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
     }
 
     if (onDemand) {
-        // use toVariant() since it could be string or bool in the json and QJsonObject does not convert
-        QVariant p = module.rawData().value(QStringLiteral("X-KDE-Kded-load-on-demand")).toVariant();
-        if (p.isValid() && (p.toBool() == false)) {
+        if (!module.value(QStringLiteral("X-KDE-Kded-load-on-demand"), true)) {
             noDemandLoad(moduleId);
             return nullptr;
         }
@@ -365,28 +355,25 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
 
     KDEDModule *kdedModule = nullptr;
 
-    KPluginLoader loader(module.fileName());
-    KPluginFactory *factory = loader.factory();
-    if (factory) {
-        kdedModule = factory->create<KDEDModule>(this);
+    auto factoryResult = KPluginFactory::loadFactory(module);
+    if (factoryResult) {
+        kdedModule = factoryResult.plugin->create<KDEDModule>(this);
     } else {
         // TODO: remove this fallback code, the kded modules should all be fixed instead
-        KPluginLoader loader2(QStringLiteral("kded_") + module.fileName());
-        factory = loader2.factory();
-        if (factory) {
-            qCWarning(KDED).nospace() << "found kded module " << moduleId
-                << " by prepending 'kded_' to the library path, please fix your metadata.";
-            kdedModule = factory->create<KDEDModule>(this);
+        factoryResult = KPluginFactory::loadFactory(KPluginMetaData(QStringLiteral("kded_") + module.fileName()));
+        if (factoryResult) {
+            qCWarning(KDED).nospace() << "found kded module " << moduleId << " by prepending 'kded_' to the library path, please fix your metadata.";
+            kdedModule = factoryResult.plugin->create<KDEDModule>(this);
         } else {
-            qCWarning(KDED).nospace() << "Could not load kded module " << moduleId << ":"
-                << loader.errorString() << " (library path was:" << module.fileName() << ")";
+            qCWarning(KDED).nospace() << "Could not load kded module " << moduleId << ":" << factoryResult.errorText
+                                      << " (library path was:" << module.fileName() << ")";
         }
     }
 
     if (kdedModule) {
         kdedModule->setModuleName(moduleId);
         m_modules.insert(moduleId, kdedModule);
-        //m_libs.insert(moduleId, lib);
+        // m_libs.insert(moduleId, lib);
         connect(kdedModule, &KDEDModule::moduleDeleted, this, &Kded::slotKDEDModuleRemoved);
         qCDebug(KDED) << "Successfully loaded module" << moduleId;
         return kdedModule;
@@ -425,12 +412,10 @@ void Kded::slotApplicationRemoved(const QString &name)
 #endif
     m_serviceWatcher->removeWatchedService(name);
     const QList<qlonglong> windowIds = m_windowIdList.value(name);
-    for (QList<qlonglong>::ConstIterator it = windowIds.begin();
-            it != windowIds.end(); ++it) {
-        qlonglong windowId = *it;
-        m_globalWindowIdList.remove(windowId);
-        for (KDEDModule *module : qAsConst(m_modules)) {
-            emit module->windowUnregistered(windowId);
+    for (const auto id : windowIds) {
+        m_globalWindowIdList.remove(id);
+        for (KDEDModule *module : std::as_const(m_modules)) {
+            Q_EMIT module->windowUnregistered(id);
         }
     }
     m_windowIdList.remove(name);
@@ -443,20 +428,23 @@ void Kded::updateDirWatch()
     }
 
     delete m_pDirWatch;
-    m_pDirWatch = new KDirWatch;
+    m_pDirWatch = new KDirWatch(this);
 
-    QObject::connect(m_pDirWatch, &KDirWatch::dirty,
-                     this, &Kded::update);
-    QObject::connect(m_pDirWatch, &KDirWatch::created,
-                     this, &Kded::update);
-    QObject::connect(m_pDirWatch, &KDirWatch::deleted,
-                     this, &Kded::dirDeleted);
+    QObject::connect(m_pDirWatch, &KDirWatch::dirty, this, &Kded::update);
+    QObject::connect(m_pDirWatch, &KDirWatch::created, this, &Kded::update);
+    QObject::connect(m_pDirWatch, &KDirWatch::deleted, this, &Kded::dirDeleted);
 
     // For each resource
-    for (QStringList::ConstIterator it = m_allResourceDirs.constBegin();
-            it != m_allResourceDirs.constEnd();
-            ++it) {
-        readDirectory(*it);
+    for (const QString &dir : std::as_const(m_allResourceDirs)) {
+        readDirectory(dir);
+    }
+
+    QStringList dataDirs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+    for (auto &dir : dataDirs) {
+        dir += QLatin1String("/icons");
+        if (!m_pDirWatch->contains(dir)) {
+            m_pDirWatch->addDir(dir, KDirWatch::WatchDirOnly);
+        }
     }
 }
 
@@ -474,12 +462,10 @@ void Kded::updateResourceList()
 
     const QStringList dirs = KSycoca::self()->allResourceDirs();
     // For each resource
-    for (QStringList::ConstIterator it = dirs.begin();
-            it != dirs.end();
-            ++it) {
-        if (!m_allResourceDirs.contains(*it)) {
-            m_allResourceDirs.append(*it);
-            readDirectory(*it);
+    for (const auto &dir : dirs) {
+        if (!m_allResourceDirs.contains(dir)) {
+            m_allResourceDirs.append(dir);
+            readDirectory(dir);
         }
     }
 }
@@ -508,7 +494,7 @@ void Kded::recreate(bool initial)
         recreateDone();
     } else {
         if (!delayedCheck) {
-            updateDirWatch();    // this would search all the directories
+            updateDirWatch(); // this would search all the directories
         }
         if (bCheckSycoca) {
             KSycoca::self()->ensureCacheValid();
@@ -537,9 +523,16 @@ void Kded::dirDeleted(const QString &path)
     update(path);
 }
 
-void Kded::update(const QString &)
+void Kded::update(const QString &path)
 {
-    m_pTimer->start(10000);
+    if (path.endsWith(QLatin1String("/icons")) && m_pDirWatch->contains(path)) {
+        // If the dir was created or updated there could be new folders to merge into the active theme(s)
+        QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KIconLoader"), QStringLiteral("org.kde.KIconLoader"), QStringLiteral("iconChanged"));
+        message << 0;
+        QDBusConnection::sessionBus().send(message);
+    } else {
+        m_pTimer->start(1000);
+    }
 }
 
 void Kded::readDirectory(const QString &_path)
@@ -549,12 +542,12 @@ void Kded::readDirectory(const QString &_path)
         path += QLatin1Char('/');
     }
 
-    if (m_pDirWatch->contains(path)) {   // Already seen this one?
+    if (m_pDirWatch->contains(path)) { // Already seen this one?
         return;
     }
 
     Q_ASSERT(path != QDir::homePath());
-    m_pDirWatch->addDir(path, KDirWatch::WatchFiles | KDirWatch::WatchSubDirs);       // add watch on this dir
+    m_pDirWatch->addDir(path, KDirWatch::WatchFiles | KDirWatch::WatchSubDirs); // add watch on this dir
 }
 
 void Kded::registerWindowId(qlonglong windowId, const QString &sender)
@@ -568,9 +561,9 @@ void Kded::registerWindowId(qlonglong windowId, const QString &sender)
     windowIds.append(windowId);
     m_windowIdList.insert(sender, windowIds);
 
-    for (KDEDModule *module : qAsConst(m_modules)) {
+    for (KDEDModule *module : std::as_const(m_modules)) {
         qCDebug(KDED) << module->moduleName();
-        emit module->windowRegistered(windowId);
+        Q_EMIT module->windowRegistered(windowId);
     }
 }
 
@@ -588,9 +581,9 @@ void Kded::unregisterWindowId(qlonglong windowId, const QString &sender)
         }
     }
 
-    for (KDEDModule *module : qAsConst(m_modules)) {
+    for (KDEDModule *module : std::as_const(m_modules)) {
         qCDebug(KDED) << module->moduleName();
-        emit module->windowUnregistered(windowId);
+        Q_EMIT module->windowUnregistered(windowId);
     }
 }
 
@@ -607,16 +600,12 @@ KUpdateD::KUpdateD()
     m_pTimer = new QTimer(this);
     m_pTimer->setSingleShot(true);
     connect(m_pTimer, &QTimer::timeout, this, &KUpdateD::runKonfUpdate);
-    QObject::connect(m_pDirWatch, &KDirWatch::dirty,
-                     this, &KUpdateD::slotNewUpdateFile);
+    QObject::connect(m_pDirWatch, &KDirWatch::dirty, this, &KUpdateD::slotNewUpdateFile);
 
-    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
-    for (QStringList::ConstIterator it = dirs.begin();
-            it != dirs.end();
-            ++it) {
-        QString path = *it;
+    QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
+    for (auto &path : dirs) {
         Q_ASSERT(path != QDir::homePath());
-        if (path[path.length() - 1] != QLatin1Char('/')) {
+        if (!path.endsWith(QLatin1Char('/'))) {
             path += QLatin1Char('/');
         }
 
@@ -678,10 +667,11 @@ static bool detectPlatform(int argc, char **argv)
         return false;
     }
     for (int i = 0; i < argc; i++) {
-        if (qstrcmp(argv[i], "-platform") == 0 ||
-                qstrcmp(argv[i], "--platform") == 0 ||
-                QByteArray(argv[i]).startsWith("-platform=") ||
-                QByteArray(argv[i]).startsWith("--platform=")) {
+        /* clang-format off */
+        if (qstrcmp(argv[i], "-platform") == 0
+            || qstrcmp(argv[i], "--platform") == 0
+            || QByteArray(argv[i]).startsWith("-platform=")
+            || QByteArray(argv[i]).startsWith("--platform=")) { /* clang-format on */
             return false;
         }
     }
@@ -689,10 +679,10 @@ static bool detectPlatform(int argc, char **argv)
     if (sessionType.isEmpty()) {
         return false;
     }
-    if (qstrcmp(sessionType, "wayland") == 0) {
+    if (qstrcmp(sessionType.data(), "wayland") == 0) {
         qputenv("QT_QPA_PLATFORM", "wayland");
         return true;
-    } else if (qstrcmp(sessionType, "x11") == 0) {
+    } else if (qstrcmp(sessionType.data(), "x11") == 0) {
         qputenv("QT_QPA_PLATFORM", "xcb");
         return true;
     }
@@ -706,7 +696,7 @@ int main(int argc, char *argv[])
     if (mainBundle) {
         // get the application's Info Dictionary. For app bundles this would live in the bundle's Info.plist,
         // for regular executables it is obtained in another way.
-        CFMutableDictionaryRef infoDict = (CFMutableDictionaryRef) CFBundleGetInfoDictionary(mainBundle);
+        CFMutableDictionaryRef infoDict = (CFMutableDictionaryRef)CFBundleGetInfoDictionary(mainBundle);
         if (infoDict) {
             // Add or set the "LSUIElement" key with/to value "1". This can simply be a CFString.
             CFDictionarySetValue(infoDict, CFSTR("LSUIElement"), CFSTR("1"));
@@ -715,7 +705,7 @@ int main(int argc, char *argv[])
         }
     }
 #endif
-    //options.add("check", qi18n("Check Sycoca database only once"));
+    // options.add("check", qi18n("Check Sycoca database only once"));
 
     // WABA: Make sure not to enable session management.
     qunsetenv("SESSION_MANAGER");
@@ -726,7 +716,7 @@ int main(int argc, char *argv[])
     // testing for --check, in which case, only a QCoreApplication was created.
     // Since that option is no longer used at startup, we removed that speed
     // optimization for code clarity and easier support of standard parameters.
-    
+
     // Fixes blurry icons with Fractional scaling
     QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
     QApplication app(argc, argv);
@@ -757,9 +747,9 @@ int main(int argc, char *argv[])
     QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
     // Also register as all the names we should respond to (org.kde.kcookiejar, org.kde.khotkeys etc.)
     // so that the calling code is independent from the physical "location" of the service.
-    const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
+    const QVector<KPluginMetaData> plugins = KPluginMetaData::findPlugins(QStringLiteral("kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/kded"));
     for (const KPluginMetaData &metaData : plugins) {
-        const QString serviceName = metaData.rawData().value(QStringLiteral("X-KDE-DBus-ServiceName")).toString();
+        const QString serviceName = metaData.value(QStringLiteral("X-KDE-DBus-ServiceName"));
         if (serviceName.isEmpty()) {
             continue;
         }
@@ -782,20 +772,15 @@ int main(int argc, char *argv[])
 
     KCrash::setFlags(KCrash::AutoRestart);
 
-    Kded *kded = new Kded();
+    std::unique_ptr<Kded> kded = std::make_unique<Kded>();
 
     kded->recreate(true); // initial
 
     if (bCheckUpdates) {
-        (void) new KUpdateD;    // Watch for updates
+        (void)new KUpdateD; // Watch for updates
     }
 
     runKonfUpdate(); // Run it once.
 
-    int result = app.exec(); // keep running
-
-    delete kded;
-
-    return result;
+    return app.exec(); // keep running
 }
-
